@@ -11,7 +11,7 @@ fn test_intraday_chart_pass() {
             let chart = fetch_enum!(Response::Chart, v);
             assert_eq!(chart.data.info.symbol_id, "2884");
         }
-        Err(e) => assert!(false, "error: {}", e.to_string()),
+        Err(e) => panic!("error: {}", e),
     };
 }
 
@@ -31,7 +31,7 @@ fn test_intraday_quote_pass() {
             let quote = fetch_enum!(Response::Quote, v);
             assert_eq!(quote.data.info.symbol_id, "2884");
         }
-        Err(e) => assert!(false, "error: {}", e.to_string()),
+        Err(e) => panic!("error: {}", e),
     };
 }
 
@@ -51,7 +51,7 @@ fn test_intraday_meta_pass() {
             let meta = fetch_enum!(Response::Meta, v);
             assert_eq!(meta.data.info.symbol_id, "2884");
         }
-        Err(e) => assert!(false, "error: {}", e.to_string()),
+        Err(e) => panic!("error: {}", e),
     };
 }
 
@@ -71,7 +71,7 @@ fn test_intraday_deals_pass() {
             let meta = fetch_enum!(Response::Dealts, v);
             assert_eq!(meta.data.info.symbol_id, "2884");
         }
-        Err(e) => assert!(false, "error: {}", e.to_string()),
+        Err(e) => panic!("error: {}", e),
     };
 }
 
@@ -82,17 +82,6 @@ fn test_intraday_dealts_failed() {
     assert!(res.is_err());
 }
 
-// the fugle api returns 403 with message Forbidden in every errors now...
-// this case was testing 401 unauthorized.
-//
-// from this time now... i don't know why 403 is not responding...
-// so this testcase cannot be tested right now...
-//
-// you can test with below command line
-// for run in {1..1000}; do curl -X GET "https://api.fugle.tw/realtime/v0.2/intraday/meta?apiToken=demo&symbolId=2884"; done
-// it was responding 403 before...
-//
-// TODO: remove the for loop when the fugle server start reponding 403
 #[test]
 fn test_error_rate_limit_exceeded() {
     let it = crawler::IntradayBuilder::new().build();
@@ -102,9 +91,8 @@ fn test_error_rate_limit_exceeded() {
             Ok(_) => continue,
             Err(e) => match e {
                 FugleError::RateLimitExceeded => break,
-                _ => assert!(false, "error: {}", e),
+                _ => panic!("error: {}", e),
             },
         }
     }
-    assert!(true)
 }
