@@ -18,7 +18,7 @@ mod util;
 #[test]
 fn test_intraday_chart_failed() {
     let (tx, _) = mpsc::channel();
-    let mut lis = listener::Intraday::new("", tx.clone());
+    let mut lis = listener::Intraday::new("", tx);
     assert!(lis.chart("2884", true).is_err());
 }
 
@@ -27,7 +27,7 @@ fn test_intraday_chart_failed() {
 fn test_intraday_chart_pass() {
     util::timeout_after(Duration::from_secs(60), || {
         let (tx, rx) = mpsc::channel();
-        let mut lis = listener::Intraday::new("demo", tx.clone());
+        let mut lis = listener::Intraday::new("demo", tx);
         assert!(lis.chart("2884", false).is_ok());
         let res = rx.recv();
         assert!(res.is_ok());
@@ -40,7 +40,7 @@ fn test_intraday_chart_pass() {
 #[test]
 fn test_intraday_meta_failed() {
     let (tx, _) = mpsc::channel();
-    let mut lis = listener::Intraday::new("", tx.clone());
+    let mut lis = listener::Intraday::new("", tx);
     assert!(lis.meta("2884", true).is_err());
 }
 
@@ -49,7 +49,7 @@ fn test_intraday_meta_failed() {
 fn test_intraday_meta_pass() {
     util::timeout_after(Duration::from_secs(60), || {
         let (tx, rx) = mpsc::channel();
-        let mut lis = listener::Intraday::new("demo", tx.clone());
+        let mut lis = listener::Intraday::new("demo", tx);
         assert!(lis.chart("2884", false).is_ok());
         let res = rx.recv();
         assert!(res.is_ok());
@@ -62,7 +62,7 @@ fn test_intraday_meta_pass() {
 #[test]
 fn test_intraday_quote_failed() {
     let (tx, _) = mpsc::channel();
-    let mut lis = listener::Intraday::new("", tx.clone());
+    let mut lis = listener::Intraday::new("", tx);
     assert!(lis.quote("2884", true).is_err());
 }
 
@@ -71,7 +71,7 @@ fn test_intraday_quote_failed() {
 fn test_intraday_quote_pass() {
     util::timeout_after(Duration::from_secs(60), || {
         let (tx, rx) = mpsc::channel();
-        let mut lis = listener::Intraday::new("demo", tx.clone());
+        let mut lis = listener::Intraday::new("demo", tx);
         assert!(lis.chart("2884", false).is_ok());
         let res = rx.recv();
         assert!(res.is_ok());
